<?php

namespace Drupal\Tests\entity_purge\Unit\Plugin\EntityPurge\Configurator;

use Drupal\entity_purge\Plugin\EntityPurge\Configurator\Age;
use Drupal\Tests\entity_purge\Unit\Configurator\PluginBaseTestTrait;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\entity_purge\Plugin\EntityPurge\Configurator\Age
 *
 * @group contrib
 * @group entity_purge
 * @group unit
 *
 * @I Write tests for entity collection
 *    type     : task
 *    priority : normal
 *    labels   : testing
 *    notes    : For better and easier testing, this might need some refactoring
 *               to use the plugin as an aggregate root and delegate query
 *               building or even entity collection to a separate class/service.
 *               Let's see how this goes before we make the code more
 *               complicated.
 */
class AgeTest extends UnitTestCase {

  use PluginBaseTestTrait {
    testAllowEntityPurgeWithDebugging as allowEntityPurgeWithDebugging;
    testAllowEntityPurgeWithoutDebugging as allowEntityPurgeWithoutDebugging;
    testDefaultConfiguration as defaultConfiguration;
    testAllowEntityPurgeWithDefaults as allowEntityPurgeWithDefaults;
  }

  use AgeTestTrait {
    testConfigurationRequiredPropertiesEntityTypeId as requiredPropertiesEntityTypeId;
    testConfigurationRequiredPropertiesFieldName as requiredPropertiesFieldName;
    testConfigurationRequiredPropertiesMinAge as requiredPropertiesMinAge;
    testConfigurationRequiredPropertiesLimit as requiredPropertiesLimit;
  }

  /**
   * Tests that the `entity_type_id` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesEntityTypeId() {
    $this->requiredPropertiesEntityTypeId(Age::class);
  }

  /**
   * Tests that the `field_name` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesFieldName() {
    $this->requiredPropertiesFieldName(Age::class);
  }

  /**
   * Tests that the `min_age` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesMinAge() {
    $this->requiredPropertiesMinAge(Age::class);
  }

  /**
   * Tests that the `limit` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesLimit() {
    $this->requiredPropertiesLimit(Age::class);
  }

  /**
   * Tests that entities are not purged when debugging is enabled.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithDebugging() {
    $this->allowEntityPurgeWithDebugging(
      Age::class,
      $this->someRequiredConfiguration(),
      $this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that entities are purged when debugging is disabled.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithoutDebugging() {
    $this->allowEntityPurgeWithoutDebugging(
      Age::class,
      $this->someRequiredConfiguration(),
      $this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that entities are purged with the default configuration.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithDefaults() {
    $this->allowEntityPurgeWithDefaults(
      Age::class,
      $this->someRequiredConfiguration(),
      $this->revealedExtraArguments(),
    );
  }

  /**
   * Tests the expected default configuration.
   *
   * @covers ::__construct
   */
  public function testDefaultConfiguration() {
    $this->defaultConfiguration();
  }

}
