<?php

namespace Drupal\Tests\entity_purge\Unit\Plugin\EntityPurge\Configurator;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_purge\Exception\InvalidConfigurationException;

/**
 * Provides tests that can be reused by all Age configurator plugins.
 *
 * These can be used to guarantee behaviors expected by plugins that extend the
 * Age configurator plugin after their customizations.
 */
trait AgeTestTrait {

  /**
   * Tests that the `entity_type_id` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesEntityTypeId(
    string $class,
  ) {
    $configuration = $this->someRequiredConfiguration();
    unset($configuration['entity_type_id']);

    $this->expectException(InvalidConfigurationException::class);
    new $class(
      $configuration,
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that the `field_name` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesFieldName(string $class) {
    $configuration = $this->someRequiredConfiguration();
    unset($configuration['field_name']);

    $this->expectException(InvalidConfigurationException::class);
    new $class(
      $configuration,
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that the `min_age` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesMinAge(string $class) {
    $configuration = $this->someRequiredConfiguration();
    unset($configuration['min_age']);

    $this->expectException(InvalidConfigurationException::class);
    new $class(
      $configuration,
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that the `limit` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesLimit(string $class) {
    $configuration = $this->someRequiredConfiguration();
    unset($configuration['limit']);

    $this->expectException(InvalidConfigurationException::class);
    new $class(
      $configuration,
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );
  }

  /**
   * Returns legitimate values for the required configuration by the Age plugin.
   */
  protected function someRequiredConfiguration(): array {
    return [
      'entity_type_id' => 'node',
      'field_name' => 'created',
      'min_age' => 86400,
      'limit' => 100,
    ];
  }

  /**
   * Returns the extra constructor arguments required by the Age plugin.
   */
  protected function revealedExtraArguments(): array {
    return [
      $this->prophesize(EntityTypeManagerInterface::class)->reveal(),
      $this->prophesize(TimeInterface::class)->reveal(),
    ];
  }

}
